require "mod-gui"
require "util"

local Functions = require "functions"
local de = defines.events
local dt = defines.train_state
local FuelValue = 10000000

script.on_init( function()
	Functions.Globals()
	Functions.Players()
end )

script.on_event( de.on_gui_click, function( event )
	if Functions.Click[event.element.name] then
		Functions.Click[event.element.name]( event )
	end
end )

script.on_event( de.on_gui_switch_state_changed, function( event )
	if Functions.Switch[event.element.name] then
		Functions.Switch[event.element.name]( event )
	end
end )

script.on_event( de.on_tick, function( event )
	if next( global.Locomotives ) and global.Accumulators > 0 then
		global.PowerTick = global.PowerTick + 1
		
		local Power = global.Power
		local Where = global.Where
		
		if next( Power["1"] ) then
			local Fuel = global.Register.Fuel[global.Fuel]
			local locomotive_table = global.Register.Locomotives
			
			for index, entity in pairs( Power["1"] ) do
				if entity.valid then
					local burner = entity.burner

					if Functions.CheckAccumulators( "S" .. entity.surface.index, FuelValue - burner.remaining_burning_fuel ) then						
						burner.currently_burning = Fuel
						burner.remaining_burning_fuel = FuelValue

						local where = locomotive_table[entity.name]

						if entity.speed ~= 0 then
							Power[where][index] = entity
							Where[index] = where
						else
							Where[index] = nil
						end
					else
						Power["2"][index] = entity
						Where[index] = "2"
					end
				else
					Where[index] = nil
					global.Locomotives[index] = nil
				end
			end
		end

		for index, loco_table in pairs( Power ) do			
			if index ~= "more" and index ~= "1" then
				index_number = "" .. tonumber( index ) - 1
				
				if next( loco_table ) then
					for unit_number in pairs( loco_table ) do
						Where[unit_number] = index_number
					end
				end

				Power[index_number] = loco_table
			end
		end

		Power["10"] = {}

		if global.PowerTick == 10 then
			global.PowerTick = 0
			
			if next( Power["more"] ) then
				for unit_number, entity in pairs( Power["more"] ) do
					local burner = entity.burner
					local FuelTick = math.floor( burner.remaining_burning_fuel / burner.heat_capacity )
					
					if FuelTick < 11 then
						if FuelTick < 1 then FuelTick = 1 end
						
						local index = "" .. FuelTick
	
						Power["more"][unit_number] = nil
						Power[index][unit_number] = entity
						Where[unit_number] = index
					end
				end
			end
		end

		global.Where = Where
		global.Power = Power
	end
end )

script.on_event( de.on_train_changed_state, function( event )
	local train = event.train
	local state = train.state
	
	if ( state == dt.wait_signal or state == dt.wait_station ) then return end

	local locomotives = train.locomotives
	local Locomotives = {}
	local locomotive_table = global.Register.Locomotives

	if next( locomotives.back_movers ) then
		for _, locomotive in pairs( locomotives.back_movers ) do
			if locomotive_table[locomotive.name] then
				Locomotives["E" .. locomotive.unit_number] = locomotive
			end
		end
	end

	if next( locomotives.front_movers ) then
		for _, locomotive in pairs( locomotives.front_movers ) do
			if locomotive_table[locomotive.name] then
				Locomotives["E" .. locomotive.unit_number] = locomotive
			end
		end
	end

	if next( Locomotives ) then
		local Power = global.Power
		local Where = global.Where

		if ( ( state == dt.arrive_signal or state == dt.arrive_station ) or ( train.speed == 0 and state ~= dt.on_the_path ) ) then
			for index, _ in pairs( Locomotives ) do
				local where = Where[index]
				
				if type( where ) == "string" then					
					Power[where][index] = nil
					Where[index] = nil
				end
			end
		else
			for index, entity in pairs( Locomotives ) do
				local burner = entity.burner
				local FuelTick = math.floor( burner.remaining_burning_fuel / burner.heat_capacity )

				if FuelTick < 10 then
					if FuelTick < 1 then FuelTick = 1 end

					local index_number = "" .. FuelTick

					Power[index_number][index] = entity
					Where[index] = index_number
				else
					Power["more"][index] = entity
					Where[index] = "more"
				end
			end
		end

		global.Power = Power
		global.Where = Where
	end
end )

script.on_event( { de.on_built_entity, de.on_robot_built_entity, de.script_raised_built, de.script_raised_revive }, function( event )
	local entity = event.created_entity or event.entity

	if entity and entity.valid then
		local unit_number = "E" .. entity.unit_number
		local name = entity.name
		local surface = entity.surface

		if global.Register.Accumulators[name] and not surface.name:find( "^Factory floor" ) then
			global.Surfaces["S" .. surface.index][unit_number] = entity
			global.Accumulators = global.Accumulators + 1
		elseif global.Register.Locomotives[name] then
			global.Locomotives[unit_number] = entity
			global.Power["1"][unit_number] = entity
			global.Where[unit_number] = "1"
		end
	end
end )

script.on_event( { de.on_surface_created, on_surface_imported }, function( event )
	global.Surfaces["S" .. event.surface_index] = global.Surfaces["S" .. event.surface_index] or {} 
end )

script.on_event( de.on_surface_deleted, function( event )
	local index = "S" .. event.surface_index
	global.Accumulators = global.Accumulators - table_size( global.Surfaces[index] )
	
	global.Surfaces[index] = nil
end )

script.on_event( de.on_research_finished, function( event )
	local name = event.research.name

	if name == "Senpais-Electric-Train-3" then
		global.Fuel = "F02"
	elseif name == "Senpais-Electric-Train-4" then
		global.Fuel = "F03"
	elseif name == "Senpais-Electric-Train-5" then
		global.Fuel = "F04"
	elseif name == "Senpais-Electric-Train-6" then
		global.Fuel = "F05"
	end
end )

script.on_event( de.on_player_created, function( event )
	Functions.PlayerStart( event.player_index )
end )

remote.add_interface
(
	"AddElectronicLocomotive",
	{
		new = function( name )
			local FuelValue = math.floor( FuelValue / ( game.entity_prototypes[name].max_energy_usage / 0.9375 ) )
			local entry = ""
			
			if FuelValue < 11 then
				entry = "" .. FuelValue
			else
				entry = "more"
			end

			global.Register.Locomotives[name] = entry
		end
	}
)