require "defines"

local MODNAME = "__Electronic_Locomotives__"
local table_deepcopy = util.table.deepcopy
local temporary01 = "Senpais-Power-Provider"
local temporary02 = MODNAME .. "/graphics/" .. temporary01 .. "-i.png"

local provider01_entity = table_deepcopy( data.raw["electric-energy-interface"]["electric-energy-interface"] )
provider01_entity.name = temporary01
provider01_entity.icon = temporary02
provider01_entity.minable.result = temporary01
provider01_entity.enable_gui = false
provider01_entity.gui_mode = "none"
provider01_entity.allow_copy_paste = false
provider01_entity.energy_source =
{
	type = "electric",
	buffer_capacity = "200MJ",
	usage_priority = "primary-input",
	input_flow_limit = "5MW",
	output_flow_limit = "0kW"
}
provider01_entity.energy_production = "0kW"
provider01_entity.energy_usage = "0kW"
provider01_entity.picture =
{
	filename = MODNAME .. "/graphics/" .. temporary01 .. "-e.png",
	priority = "extra-high",
	width = 124,
	height = 103,
	shift = { 0.6875, -0.203125 }
}
provider01_entity.charge_animation =
{
	filename = MODNAME .. "/graphics/" .. temporary01 .. "-charge.png",
	width = 138,
	height = 135,
	line_length = 8,
	frame_count = 24,
	shift = { 0.46875, -0.640625 },
	animation_speed = 0.5
}

local provider01_item = table_deepcopy( data.raw["item"]["accumulator"] )
provider01_item.name = temporary01
provider01_item.icon = temporary02
provider01_item.order = "e[accumulator]-d[" .. temporary01 .. "]"
provider01_item.place_result = temporary01

local provider01_recipe = table_deepcopy( data.raw["recipe"]["accumulator"] )
provider01_recipe.name = temporary01
provider01_recipe.ingredients =
{
	{ "accumulator", 5 },
	{ "battery", 10 },
	{ "electronic-circuit", 20 }
}
provider01_recipe.result = temporary01

local electronic_fuel =
{
	type = "item",
	icon = "__base__/graphics/icons/mip/coal.png",
	icon_size = 64,
	fuel_category = "electronic",
	fuel_value = "10MJ",
	subgroup = "raw-resource",
	order = "z[energy]",
	stack_size = 1,
	enabled = false
}

temporary02 = "electronic-fuel"

local electronic_fuel01 = table_deepcopy( electronic_fuel )
electronic_fuel01.name = temporary02 .. "-01"

local electronic_fuel02 = table_deepcopy( electronic_fuel )
electronic_fuel02.name = temporary02 .. "-02"
electronic_fuel02.fuel_acceleration_multiplier = 1.2
electronic_fuel02.fuel_top_speed_multiplier = 1.05

local electronic_fuel03 = table_deepcopy( electronic_fuel )
electronic_fuel03.name = temporary02 .. "-03"
electronic_fuel03.fuel_acceleration_multiplier = 1.8
electronic_fuel03.fuel_top_speed_multiplier = 1.15

local electronic_fuel04 = table_deepcopy( electronic_fuel )
electronic_fuel04.name = temporary02 .. "-04"
electronic_fuel04.fuel_acceleration_multiplier = 2.5
electronic_fuel04.fuel_top_speed_multiplier = 1.15

local electronic_fuel05 = table_deepcopy( electronic_fuel )
electronic_fuel05.name = temporary02 .. "-05"
electronic_fuel05.fuel_acceleration_multiplier = 3.5
electronic_fuel05.fuel_top_speed_multiplier = 1.75

temporary02 = "Senpais-Electric-Train"

local technology01 = table_deepcopy( data.raw["technology"]["railway"] )
technology01.name = temporary02
technology01.icon = MODNAME .. "/graphics/tech.png"
technology01.icon_size = 128
technology01.effects = { { type = "unlock-recipe", recipe = temporary01 } }
technology01.prerequisites = { "railway", "electric-engine", "battery", "electric-energy-distribution-2" }
technology01.unit =
{
	count = 300,
	ingredients =
	{
		{ "automation-science-pack", 2 },
		{ "logistic-science-pack", 2 },
		{ "chemical-science-pack", 1 }
	},
	time = 50
}
technology01.order = "s-e-l"

local technology02 = table_deepcopy( technology01 )
technology02.name = temporary02 .. "-2"
technology02.effects = {}
technology02.prerequisites = { temporary02 }
technology02.unit.count = 600
technology02.upgrade = true

table.insert( technology02.unit.ingredients, { "production-science-pack", 1 } )

local technology03 = table_deepcopy( technology02 )
technology03.name = temporary02 .. "-3"
technology03.effects = {}
technology03.prerequisites = { technology02.name }
technology03.unit.count = 700
technology03.localised_description = { "Electric.Description", 1.2, 1.05 }

local technology04 = table_deepcopy( technology03 )
technology04.name = temporary02 .. "-4"
technology04.effects = {}
technology04.prerequisites = { technology03.name }
technology04.unit.count = 800
technology04.localised_description = { "Electric.Description", 1.8, 1.15 }

table.insert( technology04.unit.ingredients, { "utility-science-pack", 1 } )

local technology05 = table_deepcopy( technology04 )
technology05.name = temporary02 .. "-5"
technology05.effects = {}
technology05.prerequisites = { technology04.name }
technology05.unit.count = 800
technology05.localised_description = { "Electric.Description", 2.5, 1.15 }

local technology06 = table_deepcopy( technology05 )
technology06.name = temporary02 .. "-6"
technology06.effects = {}
technology06.prerequisites = { technology05.name }
technology06.unit.count = 1200
technology06.localised_description = { "Electric.Description", 3.5, 1.75 }

table.insert( technology06.unit.ingredients, { "space-science-pack", 1 } )

data:extend
{
	{ type = "fuel-category", name = "electronic" },
	provider01_entity, provider01_item, provider01_recipe,
	electronic_fuel01, electronic_fuel02, electronic_fuel03, electronic_fuel04, electronic_fuel05,
	technology01, technology02, technology03, technology04, technology05, technology06
}

local braking_force_8 = table_deepcopy( data.raw["technology"]["braking-force-7"] )
braking_force_8.name = "braking-force-8"
braking_force_8.effects[1].modifier = 0.30
braking_force_8.prerequisites = { "braking-force-7" }
braking_force_8.unit.count = 800

local braking_force_9 = table_deepcopy( data.raw["technology"]["braking-force-7"] )
braking_force_9.name = "braking-force-9"
braking_force_9.effects[1].modifier = 0.50
braking_force_9.prerequisites = { "braking-force-8" }
braking_force_9.unit.count = 1000

data:extend{ braking_force_8, braking_force_9 }


Senpais.Functions.Create.Electronic_Locomotive
(
	temporary02,
	1000,
	2000,
	1.2,
	"600kW",
	"#53bb90",
	nil,
	"transport",
	"a[train-system]-faa[" .. temporary02 .. "]",
	5,
	{ { "locomotive", 1 }, { "battery", 10 }, { "electric-engine-unit", 20 } },
	temporary02
)

Senpais.Functions.Create.Electronic_Locomotive
(
	temporary02 .. "-Heavy",
	2000,
	5000,
	1.2,
	"3MW",
	"#a61a1a",
	nil,
	"transport",
	"a[train-system]-fab[" .. temporary02 .. "-Heavy]",
	5,
	{ { temporary02, 1 }, { "battery", 20 }, { "electric-engine-unit", 20 } },
	temporary02 .. "-2"
)

local style = data.raw["gui-style"].default

style["SenpaisFlowCenter/Left"] =
{
    type = "horizontal_flow_style",
    horizontally_stretchable = "on",
    vertical_align = "center",
    horizontal_align = "left",
    horizontal_spacing = 8
}

style["SenpaisCloseButton"] =
{
    type = "button_style",
    parent = "close_button",
    size = 28
}

style["SenpaisLine"] =
{
    type = "line_style",
    top_margin = 8,
    bottom_margin = 8
}