local Functions = {}

Functions.Watt =
{
	["watt01"] = "kW",
	["watt02"] = "MW",
	["watt03"] = "GW",
	["Watt04"] = "TW"
}

Functions.Joule =
{
	["joule01"] = "J",
	["joule02"] = "kJ",
	["joule03"] = "MJ",
	["joule04"] = "GJ",
	["joule05"] = "TJ"
}

Functions.Globals = function()
	global.Register =
	{
		Accumulators =
		{
			["Senpais-Power-Provider"] = true
		},
		Locomotives =
		{
			["Senpais-Electric-Train"] = "more",
			["Senpais-Electric-Train-Heavy"] = "more"
		},
		Fuel =
		{
			["F01"] = "electronic-fuel-01",
			["F02"] = "electronic-fuel-02",
			["F03"] = "electronic-fuel-03",
			["F04"] = "electronic-fuel-04",
			["F05"] = "electronic-fuel-05"
		}
	}
	global.Surfaces = global.Surfaces or {}
	global.Locomotives = global.Locomotives or {}
	global.GUIS = global.GUIS or {}
	global.Position = global.Position or {}
	global.Visible = global.Visible or {}
	global.UpdateLoco = global.UpdateLoco or {}
	global.Fuel = global.Fuel or "F01"
	global.Accumulators = global.Accumulators or 0
	global.PowerTick = global.PowerTick or 0
	global.Test = 0
	global.Power = global.Power or
	{
		["1"] = {}, ["2"] = {}, ["3"] = {}, ["4"] = {}, ["5"] = {},
		["6"] = {}, ["7"] = {}, ["8"] = {}, ["9"] = {}, ["10"] = {},
		["more"] = {},
	}
	global.Where = global.Where or {}

	for _, surface in pairs( game.surfaces ) do
		if not surface.name:find( "^Factory floor" ) then
			global.Surfaces["S" .. surface.index] = global.Surfaces["S" .. surface.index] or {}
		end
	end
end

Functions.Players = function()
	for _, p in pairs( game.players ) do
		Functions.PlayerStart( p.index )
	end
end

Functions.MainGUI = function( parent, player_id )
	local A = {}

	A["01"] = Functions.AddFrame( parent, "ElectricFrameAGUI01", "dialog_frame" )
	A["02"] =
	{
		["01"] = Functions.AddFlow( A["01"], "ElectricFlowAGUI01", "horizontal", "SenpaisFlowCenter/Left" ),
		["02"] = Functions.AddLine( A["01"], "ElectricLineAGUI01", "horizontal", "SenpaisLine" ),
		["03"] = Functions.AddFlow( A["01"], "ElectricFlowAGUI02", "horizontal", "SenpaisFlowCenter/Left" )
	}
	A["03"] =
	{
		["01"] = Functions.AddLabel( A["02"]["01"], "ElectricLabelAGUI01", { "Electric.Title" }, "frame_title" ),
		["02"] = Functions.AddWidget( A["02"]["01"], "ElectricWidgetAGUI01", "draggable_space_header" ),
		["03"] = Functions.AddSpriteButton( A["02"]["01"], "ElectricSpriteButtonAGUI01", "utility/close_white", "close_button" ),

		["04"] = Functions.AddLabel( A["02"]["03"], "ElectricLabelAGUI02", { "Electric.On/OffLoco" }, "caption_label" ),
		["05"] = Functions.AddWidget( A["02"]["03"], "ElectricWidgetAGUI02" ),
		["06"] = Functions.AddSwitch( A["02"]["03"], "ElectricSwitchAGUI01", { "Electric.On" }, { "Electric.Off" } ),
		["07"] = Functions.AddSpriteButton( A["02"]["03"], "ElectricSpriteButtonAGUI02", "utility/refresh", "SenpaisCloseButton" )
	}

	A["01"].location = global.Position[player_id]

	A["03"]["02"].style.horizontally_stretchable = true
	A["03"]["02"].style.natural_height = 24
	A["03"]["02"].style.minimal_width = 50
	A["03"]["02"].drag_target = A["01"]
	A["03"]["05"].style.horizontally_stretchable = true

	global.GUIS[player_id].A = A
end

Functions.MainGUIToggle = function( player_id )
	local player = game.players[player_id]
	local screen = player.gui.screen

	if screen.ElectricFrameAGUI01 then
		local GUI = global.GUIS[player_id].A["01"]
		GUI.visible = not GUI.visible
		global.Visible[player_id] = not global.Visible[player_id]
	else
		Functions.MainGUI( screen, player_id )
	end
end

Functions.PlayerStart = function( player_id )
	local player = game.players[player_id]
	local button_flow = mod_gui.get_button_flow( player )

	if not button_flow.ElectricButton then
		local b = Functions.AddSpriteButton( button_flow, "ElectricButton", "item/Senpais-Power-Provider" )
	end

	global.Position[player_id] = global.Position[player_id] or { x = 5, y = 85 * player.display_scale }
	global.GUIS[player_id] = global.GUIS[player_id] or {}
	global.Visible[player_id] = global.Visible[player_id] or false
	global.UpdateLoco[player_id] = global.UpdateLoco[player_id] or false
end

Functions.CheckAccumulators = function( index_number, amount )
	local surface_table = global.Surfaces[index_number]
	
	if next( surface_table ) then
		for index, entity in pairs( surface_table ) do
			if entity.valid then
				local energy = entity.energy

				if energy >= amount then
					entity.energy = energy - amount

					return true
				end
			else
				global.Surfaces[index_number][index] = nil
				global.Accumulators = global.Accumulators - 1
			end
		end
	end

	return false
end

Functions.AddFlow = function( parent, name, direction, style )
	return parent.add{ type = "flow", name = name, direction = direction, style = style }
end

Functions.AddFrame = function( parent, name, style )
	return parent.add{ type = "frame", name = name, direction = "vertical", style = style }
end

Functions.AddLabel = function( parent, name, caption, style )
	return parent.add{ type = "label", name = name, caption = caption, style = style }
end

Functions.AddLine = function( parent, name, direction, style )
	return parent.add{ type = "line", name = name, direction = direction, style = style }
end

Functions.AddSpriteButton = function( parent, name, sprite, style )
	return parent.add{ type = "sprite-button", name = name, sprite = sprite, style = style }
end

Functions.AddSwitch = function( parent, name, left_caption, right_caption )
	return parent.add{ type = "switch", name = name, switch_state = "right", left_label_caption = left_caption, right_label_caption = right_caption }
end

Functions.AddWidget = function( parent, name, style )
	return parent.add{ type = "empty-widget", name = name, style = style }
end

local click = {}

click["ElectricButton"] = function( event )
	Functions.MainGUIToggle( event.player_index )
end

click["ElectricSpriteButtonAGUI01"] = function( event )
	local player_id = event.player_index
	
	global.GUIS[player_id].A["01"].visible = false
	global.Visible[player_id] = false
end

Functions.Click = click

local switch = {}

switch["ElectricSwitchAGUI01"] = function( event )
	global.Test = global.Test + 1

	game.print( global.Test )
	--[[local boolean = false

	if event.element.switch_state == "left" then
		boolean = true
	end

	global.UpdateLoco[event.player_index] = boolean]]
end

Functions.Switch = switch


return Functions